package escape;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

public class DoorMode extends GameMode {

  public static final int LEFT_REGION = 100;
  public static final int RIGHT_REGION = 101;
  public static final int UP_REGION = 102;
  public static final int DOWN_REGION = 103;

  public static final int PICK_REGION = 104;
  public static final int DOOR_REGION = 105;
  public static final int EXIT_REGION = 106;

  public boolean crankTurning;
  public int crankIndex;
  public int crankIndexIncrementor;
  public int paperTowelY = 294;

  public DoorMode() {
    addClickRegions();
  }

  public void addClickRegions() {
    clickRegions.clear();
    if (GameState.doorOpened) {

    } else {
      if (GameState.doorUnlocked) {
        clickRegions.add(new Region(517, 283, 15, 41, DOOR_REGION));
      }
      clickRegions.add(new Region(521, 325, 7, 14, 1));
    }
    if (!GameState.acquiredPenny) {
      clickRegions.add(new Region(506, 495, 10, 5, 0));
    }
    if (!GameState.turnedCrank) {
      clickRegions.add(new Region(288, 273, 56, 37, 2));
    }
    if (GameState.turnedCrank && !GameState.acquiredPaperTowel) {
      clickRegions.add(new Region(196, 371, 87, 73, 3));
    }

    clickRegions.add(new Region(0, 80, 60, 520, LEFT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(740, 80, 60, 520, RIGHT_REGION,
        Images.arrowCursors[Images.RIGHT]));
    if (GameState.doorOpened) {
      clickRegions.add(new Region(40, 80, 420, 100, UP_REGION,
          Images.arrowCursors[Images.UP]));
      clickRegions.add(new Region(502, 100, 169, 395, EXIT_REGION,
          Images.arrowCursors[Images.UP]));
    } else {
      clickRegions.add(new Region(40, 80, 700, 100, UP_REGION,
          Images.arrowCursors[Images.UP]));
    }
    clickRegions.add(new Region(40, 520, 700, 80, DOWN_REGION,
        Images.arrowCursors[Images.DOWN]));
  }

  @Override public void modify() {
    if (crankTurning) {
      if (--crankIndexIncrementor < 0) {
        crankIndexIncrementor = 6;
        if (++crankIndex == 4) {
          crankIndex = 0;
        }
      }
      if (++paperTowelY == 371) {
        crankIndex = 0;
        crankTurning = false;
        addClickRegions();
      }
    }
  }

  @Override public void draw(final Graphics2D g) {
    if (GameState.doorOpened) {
      g.drawImage(Images.openedDoor, 0, 80, null);
    } else {
      g.drawImage(Images.doorBackground, 0, 80, null);
    }
    if (!GameState.acquiredPenny) {
      g.drawImage(Images.pennies[1], 506, 495, null);
    }
    if (GameState.turnedCrank && !GameState.acquiredPaperTowel) {
      g.drawImage(Images.paperTowels[1], 196, paperTowelY, null);
    }
    g.drawImage(Images.paperTowelDispenser, 184, 259, null);
    g.drawImage(Images.cranks[crankIndex], 288, 273, null);
  }

  @Override public void regionClicked(final Region clickRegion) {
    switch(clickRegion.id) {
      case EXIT_REGION:
        main.setMode(main.endingMode);
        break;
      case DOOR_REGION:
        GameState.doorOpened = true;
        addClickRegions();
        break;
      case LEFT_REGION:
        main.setMode(main.trashCanMode);
        addClickRegions();
        break;
      case RIGHT_REGION:
        main.setMode(main.stallsMode);
        addClickRegions();
        break;
      case UP_REGION:
        main.setMode(main.smokeAlarmMode);
        addClickRegions();
        break;
      case DOWN_REGION:
        main.setMode(main.drainMode);
        addClickRegions();
        break;
      case 0:
        GameState.acquiredPenny = true;
        addToInventory(Images.pennies[0]);
        addClickRegions();
        break;
      case 1:
        Main.main.keyholeMode.reset();
        Main.main.setMode(Main.main.keyholeMode);
        addClickRegions();
        break;
      case 2:
        crankTurning = true;
        GameState.turnedCrank = true;
        break;
      case 3:
        GameState.acquiredPaperTowel = true;
        addToInventory(Images.paperTowels[2]);
        addClickRegions();
        break;
    }
  }

  @Override
  public void inventoryItemLifted(final BufferedImage image) {
    if (image == Images.picks[2]) {
      dropRegions.add(new Region(521, 325, 7, 14, PICK_REGION));
    }
  }

  @Override
  public void regionDropped(final BufferedImage image, 
      final Region dropRegion) {
    
    if (image == Images.picks[2]) {
      main.setMode(main.lockMode);
    }
  }

  @Override
  public boolean isInventoryAccessible() {
    return !crankTurning;
  }

  @Override
  public void enter() {
    super.enter();
    addClickRegions();
  }
}