package escape;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

public abstract class GameMode implements IMode {

  public static final Random random = new Random();

  public final Main main;

  public final ArrayList<Region> clickRegions = new ArrayList<>();
  public final ArrayList<Region> dropRegions = new ArrayList<>();
  public final Region[] inventoryRegions = new Region[10];

  public Region visibleRegion;
  public float visibleRegionAngle;
  public boolean mousePressed;
  public BufferedImage draggedImage;
  public int draggedOffsetX;
  public int draggedOffsetY;
  public boolean draggingInventoryItem;
  public MouseCursor mouseCursor;

  public GameMode() {
    main = Main.main;
    for(int i = 0; i < 10; i++) {
      inventoryRegions[i] = new Region(5 + i * 80, 5, 70, 70);
    }
  }

  @Override public final void update() {
    visibleRegion = null;
    visibleRegionAngle += 0.1f;

    if (draggedImage == null) {

      if (isInventoryAccessible()) {
        for(int i = 0; i < 10; i++) {
          if (Mouse.y >= 0 && Mouse.y <= 80 && GameState.inventory[i] != null
              && Mouse.x >= i * 80 && Mouse.x <= (i + 1) * 80) {
            visibleRegion = inventoryRegions[i];
            if (Mouse.pressed && mousePressed != Mouse.pressed) {
              dropRegions.clear();
              draggingInventoryItem = true;
              inventoryItemLifted(GameState.inventory[i]);
              dragImage(GameState.inventory[i], Mouse.x - i * 80, Mouse.y);
              GameState.inventory[i] = null;              
              for(int j = 0; j < 10; j++) {
                if (GameState.inventory[j] == null) {
                  dropRegions.add(new Region(5 + j * 80, 5, 70, 70, 1000 + j));
                }
              }
              addDropRegions();
            }
            break;
          }
        }
      }

      if (visibleRegion == null) {
        for(int i = clickRegions.size() - 1; i >= 0; i--) {
          Region clickRegion = clickRegions.get(i);
          if (Mouse.x >= clickRegion.x && Mouse.y >= clickRegion.y
              && Mouse.x <= clickRegion.x + clickRegion.width
              && Mouse.y <= clickRegion.y + clickRegion.height) {
            visibleRegion = clickRegion;
            if (visibleRegion.mouseCursor != null && Mouse.visible) {
              main.hideMouseCursor();
            }
            if (Mouse.pressed && mousePressed != Mouse.pressed) {
              clickRegions.clear();
              regionClicked(clickRegion);
            }
            break;
          }
        }
      }

      if (visibleRegion == null && !Mouse.visible && mouseCursor == null) {
        main.showMouseCursor();
      }      
    } else {
      for(int i = dropRegions.size() - 1; i >= 0; i--) {
        Region dropRegion = dropRegions.get(i);
        if (Mouse.x >= dropRegion.x
            && Mouse.y >= dropRegion.y
            && Mouse.x <= dropRegion.x + dropRegion.width
            && Mouse.y <= dropRegion.y + dropRegion.height) {
          visibleRegion = dropRegion;
          break;
        }
      }

      if (visibleRegion == null) {

        int x1 = Mouse.x - draggedOffsetX;
        int y1 = Mouse.y - draggedOffsetY;
        int x2 = x1 + draggedImage.getWidth();
        int y2 = y1 + draggedImage.getHeight();

        for(int i = dropRegions.size() - 1; i >= 0; i--) {
          Region dropRegion = dropRegions.get(i);
          if (dropRegion.x <= x2 && dropRegion.x + dropRegion.width >= x1
              && dropRegion.y <= y2 && dropRegion.y + dropRegion.height >= y1) {
            visibleRegion = dropRegion;
            break;
          }
        }
      }

      if (!Mouse.pressed) {
        final BufferedImage _draggedImage = draggedImage;
        draggedImage = null;
        dropRegions.clear();
        if (mouseCursor == null) {
          main.showMouseCursor();
        }

        if (visibleRegion == null && draggingInventoryItem) {          
          for(int i = 0; i < 10; i++) {
            if (GameState.inventory[i] == null) {
              GameState.inventory[i] = _draggedImage;
              break;
            }
          }
        } else if (visibleRegion != null && visibleRegion.id >= 1000) {
          GameState.inventory[visibleRegion.id - 1000] = _draggedImage;
        } else if (!handleDropRegion(_draggedImage, visibleRegion)) {
          regionDropped(_draggedImage, visibleRegion);
        }
        
        draggingInventoryItem = false;
      }
    }

    mousePressed = Mouse.pressed;
    modify();
  }

  @Override public final void render(final Graphics2D g) {
    for(int i = 0; i < 10; i++) {
      g.drawImage(Images.inventoryTile, i * 80, 0, null);
      if (GameState.inventory[i] != null) {
        g.drawImage(GameState.inventory[i], i * 80, 0, null);
      }
    }

    draw(g);

    MouseCursor cursor = mouseCursor;
    if (visibleRegion != null) {
      if (visibleRegion.mouseCursor == null) {
        float offset = (float)Math.sin(visibleRegionAngle);
        int o = (int)(4 * offset * offset);
        g.setColor(Color.GREEN);
        g.drawRect(visibleRegion.x - o, visibleRegion.y - o,
            visibleRegion.width + (o << 1), visibleRegion.height + (o << 1));
      } else {
        cursor = visibleRegion.mouseCursor;        
      }
    }

    if (draggedImage != null) {
      g.drawImage(draggedImage, Mouse.x - draggedOffsetX,
          Mouse.y - draggedOffsetY, null);
    } else if (cursor != null) {
      g.drawImage(cursor.image,
          Mouse.x - cursor.hotspotX, Mouse.y - cursor.hotspotY,
          null);
    }
  }

  public void dragImage(final BufferedImage draggedImage,
      final int draggedOffsetX, final int draggedOffsetY) {
    
    this.draggedImage = draggedImage;
    this.draggedOffsetX = draggedOffsetX;
    this.draggedOffsetY = draggedOffsetY;
    main.hideMouseCursor();
  }

  public void removeFromInventory(final BufferedImage itemImage) {
    for(int i = 0; i < 10; i++) {
      if (GameState.inventory[i] == itemImage) {
        GameState.inventory[i] = null;
        break;
      }
    }
  }

  public void addToInventory(final BufferedImage itemImage) {
    for(int i = 0; i < 10; i++) {
      if (GameState.inventory[i] == null) {
        GameState.inventory[i] = itemImage;
        break;
      }
    }
  }

  public void setMouseCursor(final MouseCursor mouseCursor) {
    this.mouseCursor = mouseCursor;
    main.hideMouseCursor();
  }

  private void addDropRegions() {
    if (draggedImage == Images.paperTowels[2]
        || draggedImage == Images.printouts[1]
        || draggedImage == Images.toiletPapers[3]) {
      dropRegions.add(new Region(10, 90, 780, 500));
    } else if (draggedImage == Images.magnets[0]) {
      addDropRegion(Images.shoeLaces[0]);
    } else if (draggedImage == Images.shoeLaces[0]) {
      addDropRegion(Images.magnets[0]);
    }
  }

  public void addDropRegion(final BufferedImage itemImage) {
    for(int i = 0; i < 10; i++) {
      if (GameState.inventory[i] == itemImage) {
        dropRegions.add(new Region(5 + i * 80, 5, 70, 70));
        break;
      }
    }
  }

  private boolean handleDropRegion(final BufferedImage item, 
      final Region region) {
    
    if (item == Images.toiletPapers[3]) {
      main.pushMode(main.toiletPaperMode);
      return true;
    } else if (item == Images.paperTowels[2]) {
      main.pushMode(main.paperTowelMode);
      return true;
    } else if (item == Images.printouts[1]) {
      main.pushMode(main.printoutMode);
      return true;
    } else if (item == Images.magnets[0] || item == Images.shoeLaces[0]) {
      removeFromInventory(Images.magnets[0]);
      removeFromInventory(Images.shoeLaces[0]);
      main.pushMode(main.connectMode);
      return true;
    }
    return false;
  }

  @Override
  public void enter() {
    mousePressed = Mouse.pressed;
  }

  public abstract void modify();
  public abstract void draw(final Graphics2D g);
  
  public void regionClicked(final Region clickRegion) {
  }

  public void regionDropped(final BufferedImage image, final Region dropRegion){
  }

  public void inventoryItemLifted(final BufferedImage image) {
  }

  public boolean isInventoryAccessible() {
    return true;
  }
}
