package escape;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

public class StallMode extends GameMode {

  public static final int STATE_START = 0;
  public static final int STATE_POURING_1 = 1;
  public static final int STATE_POURING_2 = 2;
  public static final int STATE_POURING_3 = 3;
  public static final int STATE_POURING_4 = 4;
  public static final int STATE_POURING_5 = 5;
  public static final int STATE_OPENING = 1;

  public static final int REGION_TOOTH = 0;
  public static final int REGION_SHOELACE = 1;
  public static final int REGION_EXIT = 2;
  public static final int REGION_UNROLL = 3;
  public static final int REGION_TAKE_TOILET_PAPER = 4;
  public static final int REGION_KEYPAD = 5;
  public static final int REGION_PLUNGER = 6;
  public static final int REGION_DOOR = 7;
  public static final int REGION_MAGNET = 8;

  private static final Color[] colors = {
    new Color(0xFF0000),
    new Color(0xCCFF00),
    new Color(0x00FF66),
    new Color(0x0066FF),
    new Color(0xCC00FF),
    new Color(0xFFFFFF),
  };

  private int[] combination = new int[7];

  private boolean isLockedStall;
  private boolean isToiletPaperStall;
  private boolean isPlungerStall;

  private int state = STATE_START;
  private int pourHeight;
  private int pouringIndex;
  private int pouringIndexIncrementor;
  private int delay;
  private int monsterIndex;
  private int monsterIndexIncrementor;  
  private int doorHeight;
  private int doorTargetHeight;
  
  public StallMode() {
    addClickRegions();

    combination[0] = 5;
    for(int i = 1; i < 7; i++) {
      combination[i] = GameState.lockCombination[i - 1];
    }
  }

  private void addClickRegions() {
    clickRegions.clear();
    if (state == STATE_START) {
      if (isLockedStall) {
        clickRegions.add(new Region(0, 520, 268, 80, REGION_EXIT,
            Images.arrowCursors[Images.DOWN]));
        clickRegions.add(new Region(268, 560, 89, 40, REGION_EXIT,
            Images.arrowCursors[Images.DOWN]));
        clickRegions.add(new Region(357, 520, 443, 80, REGION_EXIT,
            Images.arrowCursors[Images.DOWN]));
      } else {
        clickRegions.add(new Region(0, 520, 800, 80, REGION_EXIT,
            Images.arrowCursors[Images.DOWN]));
      }
      if (isLockedStall) {
        if (GameState.killedToiletCreature && !GameState.acquiredTooth) {
          clickRegions.add(new Region(358, 295, 94, 69, REGION_TOOTH));
        }
        if (!GameState.acquiredShoelace) {
          clickRegions.add(new Region(268, 482, 89, 67, REGION_SHOELACE));
        }
      } else if (isToiletPaperStall) {
        if (!GameState.acquiredToiletPaper && !GameState.unrolledToiletPaper) {
          clickRegions.add(new Region(137, 164, 76, 47, REGION_UNROLL));
        }
        if (!GameState.acquiredToiletPaper && GameState.unrolledToiletPaper) {
          clickRegions.add(new Region(139, 220, 75, 61,
              REGION_TAKE_TOILET_PAPER));
        }
      } else {
        if (!GameState.passwordEntered) {
          clickRegions.add(new Region(378, 118, 45, 50, REGION_KEYPAD));
        } else if (!GameState.plungerSwitched) {
          clickRegions.add(new Region(89, 217, 32, 230, REGION_PLUNGER));
        } else if (GameState.doorsOpened < 6) {
          clickRegions.add(new Region(105, 96, 164, 282, REGION_DOOR));
        } else if (!GameState.acquiredMagnet) {
          clickRegions.add(new Region(151, 317, 80, 60, REGION_MAGNET));
        }
      }
    }
  }

  @Override
  public void modify() {
    if (isLockedStall) {
      if (state == STATE_POURING_1) {
        if (pourHeight < 200) {
          pourHeight += 10;
        } else {
          state = STATE_POURING_2;
          delay = 0;
        }
      } else if (state == STATE_POURING_2) {
        if (++pouringIndexIncrementor == 5) {
          pouringIndexIncrementor = 0;
          if (++pouringIndex == 2) {
            pouringIndex = 0;
          }
        }
        if (++delay == 60) {
          delay = 0;
          state = STATE_POURING_3;
        }
      } else if (state == STATE_POURING_3) {
        if (++pouringIndexIncrementor == 5) {
          pouringIndexIncrementor = 0;
          if (++pouringIndex == 2) {
            pouringIndex = 0;
          }
        }
        if (++monsterIndexIncrementor == 7) {
          monsterIndexIncrementor = 0;
          if (++monsterIndex == 2) {
            monsterIndex = 0;
          }
        }
        if (++delay == 90) {
          delay = 0;
          state = STATE_POURING_4;
        }
      } else if (state == STATE_POURING_4) {
        if (++pouringIndexIncrementor == 5) {
          pouringIndexIncrementor = 0;
          if (++pouringIndex == 2) {
            pouringIndex = 0;
          }
        }
        if (++monsterIndexIncrementor == 7) {
          monsterIndexIncrementor = 0;
          if (++monsterIndex == 2) {
            monsterIndex = 0;
          }
        }
        if (pourHeight > 0) {
          pourHeight -= 10;
        } else {
          state = STATE_POURING_5;
        }
      } else if (state == STATE_POURING_5) {
        if (++monsterIndexIncrementor == 7) {
          monsterIndexIncrementor = 0;
          if (++monsterIndex == 2) {
            monsterIndex = 0;
          }
        }
        if (++delay == 70) {
          delay = 0;
          GameState.killedToiletCreature = true;
          state = STATE_START;
          addClickRegions();
        }
      }
    } else if (isPlungerStall) {
      if (state == STATE_OPENING) {
        if (doorHeight > doorTargetHeight) {
          doorHeight -= 2;
        } else {
          state = STATE_START;
          addClickRegions();
        }
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.drawImage(Images.stall, 0, 80, null);

    if (isLockedStall) {
      g.drawImage(Images.shoes[GameState.acquiredShoelace ? 1 : 0],
          243, 462, null);
      g.drawImage(Images.shoes[2], 447, 462, null);
      if (state == STATE_START) {
        if (GameState.killedToiletCreature) {
          if (GameState.acquiredTooth) {
            g.drawImage(Images.toiletMonsters[3], 300, 214, null);
          } else {
            g.drawImage(Images.toiletMonsters[2], 300, 214, null);
          }
        } else {
          g.drawImage(Images.toiletMonsters[0], 300, 214, null);
        }
      } else if (state == STATE_POURING_1) {
        g.drawImage(Images.toiletMonsters[0], 300, 214, null);
        g.setClip(378, 151, 48, pourHeight);
        g.drawImage(Images.pouring[0], 378, 151, null);
        g.setClip(null);
        g.drawImage(Images.bottles[2], 412, 82, null);
      } else if (state == STATE_POURING_2) {
        g.drawImage(Images.toiletMonsters[0], 300, 214, null);
        g.drawImage(Images.pouring[pouringIndex], 378, 151, null);
        g.drawImage(Images.bottles[2], 412, 82, null);
      } else if (state == STATE_POURING_3) {
        g.drawImage(Images.toiletMonsters[monsterIndex], 300, 214, null);
        g.drawImage(Images.pouring[pouringIndex], 378, 151, null);
        g.drawImage(Images.bottles[2], 412, 82, null);
      } else if (state == STATE_POURING_4) {
        g.drawImage(Images.toiletMonsters[monsterIndex], 300, 214, null);
        g.setClip(378, 351 - pourHeight, 48, pourHeight);
        g.drawImage(Images.pouring[0], 378, 151, null);
        g.setClip(null);
      } else if (state == STATE_POURING_5) {
        g.drawImage(Images.toiletMonsters[monsterIndex], 300, 214, null);
      }
    } else if (isToiletPaperStall) {
      g.drawImage(Images.toiletPapers[GameState.unrolledToiletPaper
          && !GameState.acquiredToiletPaper ? 1 : 0], 118, 159, null);
    } else {
      g.drawImage(Images.keypads[GameState.batteryInstalled
          ? GameState.passwordEntered ? 2 : 1 : 0], 374, 82, null);

      if (GameState.plungerSwitched) {
        g.drawImage(Images.plungers[1], 34, 217, null);

        g.setClip(106, 97, 162, 280);        
        g.setColor(colors[combination[GameState.doorsOpened]]);
        g.fillRect(106, 97, 162, 280);
        if (GameState.doorsOpened < 6) {
          g.drawImage(Images.doorHandle, 175, 358, null);
        } else {
          g.drawImage(Images.magnetBox, 106, 178, null);
          if (!GameState.acquiredMagnet) {
            g.drawImage(Images.magnets[0], 151, 307, null);
          }
        }
        for(int i = GameState.doorsOpened - 1; i >= 0; i--) {
          if (i == GameState.doorsOpened - 1) {
            g.setColor(colors[combination[i]]);
            g.fillRect(106, 97, 162, doorHeight);
            g.setColor(Color.BLACK);
            g.drawRect(106, 97, 162, doorHeight);
            if (i != 0) {
              g.drawImage(Images.doorHandle, 175, 78 + doorHeight, null);
            }
          } else {
            g.setColor(colors[combination[i]]);
            g.fillRect(106, 97, 162, i << 4);
            g.setColor(Color.BLACK);
            g.drawRect(106, 97, 162, i << 4);
            g.drawImage(Images.doorHandle, 175, 78 + (i << 4), null);
          }
        }
        g.setClip(null);

        g.setColor(Color.BLACK);
        g.drawRect(106, 97, 162, 280);

      } else {
        g.drawImage(Images.plungers[0], 34, 217, null);
      }
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {
    if (clickRegion.id == REGION_MAGNET) {
      GameState.acquiredMagnet = true;
      addToInventory(Images.magnets[0]);
    } else if (clickRegion.id == REGION_DOOR) {
      state = STATE_OPENING;
      doorHeight = 280;
      doorTargetHeight = GameState.doorsOpened << 4;
      GameState.doorsOpened++;
    } else if (clickRegion.id == REGION_KEYPAD) {
      main.pushMode(main.keypadMode);
    } else if (clickRegion.id == REGION_TAKE_TOILET_PAPER) {
      addToInventory(Images.toiletPapers[3]);
      GameState.acquiredToiletPaper = true;
    } else if (clickRegion.id == REGION_EXIT) {
      main.setMode(main.stallsMode);
    } else if (clickRegion.id == REGION_TOOTH) {
      addToInventory(Images.tooth);
      GameState.acquiredTooth = true;
    } else if (clickRegion.id == REGION_SHOELACE) {
      addToInventory(Images.shoeLaces[0]);
      GameState.acquiredShoelace = true;
    } else if (clickRegion.id == REGION_UNROLL) {
      GameState.unrolledToiletPaper = true;
    } else if (clickRegion.id == REGION_PLUNGER) {
      GameState.plungerSwitched = true;
      state = STATE_OPENING;
      doorHeight = 280;
      doorTargetHeight = 0;
      GameState.doorsOpened = 1;
    }
    addClickRegions();
  }

  @Override
  public void regionDropped(BufferedImage image, Region dropRegion) {
    if (isLockedStall) {
      if (image == Images.bottles[1]) {
        state = STATE_POURING_1;
        clickRegions.clear();
      }
    } else if (isPlungerStall) {
      if (image == Images.battery) {
        GameState.batteryInstalled = true;
      }
    }
  }

  public void setIndex(int index) {
    isLockedStall = GameState.lockedStallIndex == index;
    isToiletPaperStall = GameState.toiletPaperStallIndex == index;
    isPlungerStall = GameState.plungerStallIndex == index;
    addClickRegions();
  }

  @Override
  public void inventoryItemLifted(BufferedImage image) {
    if (isLockedStall) {
      if (image == Images.bottles[1]) {
        dropRegions.add(new Region(300, 214, 197, 178));
      }
    } else if (isPlungerStall) {
      if (image == Images.battery) {
        dropRegions.add(new Region(381, 169, 39, 23));
      }
    }
  }

  @Override
  public boolean isInventoryAccessible() {
    return state == STATE_START;
  }

  @Override
  public void enter() {
    super.enter();
    addClickRegions();
  }
}
