package escape;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

public class StallsMode extends GameMode {

  public static final int REGION_LEFT = 0;
  public static final int REGION_RIGHT = 1;
  public static final int REGION_UP = 2;
  public static final int REGION_DOWN = 3;
  public static final int REGION_PENNY = 4;
  public static final int REGION_LEFT_STALL = 5;
  public static final int REGION_MIDDLE_STALL = 6;
  public static final int REGION_RIGHT_STALL = 7;
  
  public StallsMode() {
    addClickRegions();
  }

  private void addClickRegions() {
    clickRegions.clear();
    clickRegions.add(new Region(0, 80, 40, 520, REGION_LEFT,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(750, 80, 50, 520, REGION_RIGHT,
        Images.arrowCursors[Images.RIGHT]));
    clickRegions.add(new Region(40, 80, 700, 50, REGION_UP,
        Images.arrowCursors[Images.UP]));
    clickRegions.add(new Region(40, 550, 700, 50, REGION_DOWN,
        Images.arrowCursors[Images.DOWN]));

    if (GameState.lockedStallIndex != 0 || GameState.stallUnlocked) {
      clickRegions.add(new Region(57, 160, 185, 339, REGION_LEFT_STALL,
          Images.arrowCursors[Images.UP]));
    }
    if (GameState.lockedStallIndex != 1 || GameState.stallUnlocked) {
      clickRegions.add(new Region(305, 160, 185, 339, REGION_MIDDLE_STALL,
          Images.arrowCursors[Images.UP]));
    }
    if (GameState.lockedStallIndex != 2 || GameState.stallUnlocked) {
      clickRegions.add(new Region(553, 160, 185, 339, REGION_RIGHT_STALL,
          Images.arrowCursors[Images.UP]));
    }
  }

  @Override
  public void modify() {
  }

  @Override
  public void draw(Graphics2D g) {
    g.drawImage(Images.stallsBackground, 0, 80, null);

    for(int i = 0; i < 4; i++) {
      g.drawImage(Images.stallDivider, 248 * i - 12, 80, null);
    }

    if (GameState.stallUnlocked) {
      for(int i = 0; i < 3; i++) {
        if (i == GameState.lockedStallIndex) {
          g.drawImage(Images.shoes[GameState.acquiredShoelace ? 4 : 3],
              248 * i + 100, 502, null);
          g.drawImage(Images.shoes[5], 248 * i + 170, 502, null);
        }
        g.drawImage(Images.stallDoors[1], 248 * i + 57, 140, null);
      }
    } else {
      for(int i = 0; i < 3; i++) {
        if (i == GameState.lockedStallIndex) {
          g.drawImage(Images.shoes[3], 248 * i + 100, 502, null);
          g.drawImage(Images.shoes[5], 248 * i + 170, 502, null);
          g.drawImage(Images.stallDoors[0], 248 * i + 57, 140, null);
        } else {
          g.drawImage(Images.stallDoors[1], 248 * i + 57, 140, null);
        }
      }
    }
  }

  @Override
  public void inventoryItemLifted(BufferedImage image) {
    if (image == Images.pennies[0]) {
      dropRegions.add(new Region(248 * GameState.lockedStallIndex + 46,
          310, 45, 48, REGION_PENNY));
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {
    if (clickRegion.id == REGION_LEFT_STALL) {
      main.stallMode.setIndex(0);
      main.setMode(main.stallMode);
    } else if (clickRegion.id == REGION_MIDDLE_STALL) {
      main.stallMode.setIndex(1);
      main.setMode(main.stallMode);
    } else if (clickRegion.id == REGION_RIGHT_STALL) {
      main.stallMode.setIndex(2);
      main.setMode(main.stallMode);
    } else if (clickRegion.id == REGION_LEFT) {
      main.setMode(main.doorMode);
    } else if (clickRegion.id == REGION_RIGHT) {
      main.setMode(main.closetMode);
    } else if (clickRegion.id == REGION_UP) {
      main.setMode(main.smokeAlarmMode);
    } else if (clickRegion.id == REGION_DOWN) {
      main.setMode(main.drainMode);
    }
    addClickRegions();
  }

  @Override
  public void regionDropped(BufferedImage image, Region dropRegion) {
    if (image == Images.pennies[0]) {
      main.setMode(main.pennyUnlockMode);
    }
  }

  @Override
  public void enter() {
    super.enter();
    addClickRegions();
  }
}
