package escape;

import java.awt.*;

public class TileMode extends GameMode {

  public static final int EXIT_REGION = 10;

  private final Color GRAY = new Color(0xD5DCE0);

  public int[][] tiles = new int[4][4];
  public int holeX = 3;
  public int holeY = 3;

  public int movingSteps;
  public int movingTile;
  public int movingX;
  public int movingY;
  public int movingVx;
  public int movingVy;
  public int targetX;
  public int targetY;

  private int solvedCountdown;

  public TileMode() {
    for(int y = 0; y < 4; y++) {
      for(int x = 0; x < 4; x++) {
        tiles[y][x] = 4 * y + x;
      }
    }
    do {
      for(int i = 0; i < 2048; i++) {
        switch(random.nextInt(4)) {
          case Images.UP:
            if (swap(holeX, holeY, holeX, holeY - 1)) {
              holeY--;
            } else {
              i--;
            }
            break;
          case Images.DOWN:
            if (swap(holeX, holeY, holeX, holeY + 1)) {
              holeY++;
            } else {
              i--;
            }
            break;
          case Images.LEFT:
            if (swap(holeX - 1, holeY, holeX, holeY)) {
              holeX--;
            } else {
              i--;
            }
            break;
          case Images.RIGHT:
            if (swap(holeX + 1, holeY, holeX, holeY)) {
              holeX++;
            } else {
              i--;
            }
            break;
        }
      }
    } while(isSolved());
    addClickRegions();
  }

  private boolean isSolved() {
    for(int y = 0; y < 4; y++) {
      for(int x = 0; x < 4; x++) {
        if (tiles[y][x] != (y << 2) + x) {
          return false;
        }
      }
    }
    return true;
  }

  private void addClickRegions() {
    if (holeX > 0) {
      clickRegions.add(new Region(
          140 + (holeX - 1) * 130, 80 + holeY * 130, 130, 130, Images.LEFT));
    }
    if (holeY > 0) {
      clickRegions.add(new Region(
          140 + holeX * 130, 80 + (holeY - 1) * 130, 130, 130, Images.UP));
    }
    if (holeX < 3) {
      clickRegions.add(new Region(
          140 + (holeX + 1) * 130, 80 + holeY * 130, 130, 130, Images.RIGHT));
    }
    if (holeY < 3) {
      clickRegions.add(new Region(
          140 + holeX * 130, 80 + (holeY + 1) * 130, 130, 130, Images.DOWN));
    }

    clickRegions.add(new Region(0, 80, 100, 520, EXIT_REGION,
        Images.arrowCursors[Images.LEFT]));
    clickRegions.add(new Region(700, 80, 100, 520, EXIT_REGION,
        Images.arrowCursors[Images.RIGHT]));
  }

  private boolean swap(int x1, int y1, int x2, int y2) {
    if (x1 < 0 || y1 < 0 || x2 < 0 || y2 < 0
        || x1 > 3 || y1 > 3 || x2 > 3 || y2 > 3) {
      return false;
    }
    int temp = tiles[y2][x2];
    tiles[y2][x2] = tiles[y1][x1];
    tiles[y1][x1] = temp;
    return true;
  }

  @Override
  public void modify() {
    if (solvedCountdown > 0) {
      if (--solvedCountdown == 0) {
        main.setMode(main.closetMode);
      }
    } else if (movingSteps > 0) {
      movingX += movingVx;
      movingY += movingVy;
      if (--movingSteps == 0) {
        tiles[targetY][targetX] = movingTile;
        if (isSolved()) {
          GameState.tilePuzzleSolved = true;
          solvedCountdown = 180;
          main.closetMode.solvedCountdown = 60;
          main.closetMode.clickRegions.clear();
        } else {
          addClickRegions();
        }
      }
    }
  }

  @Override
  public void draw(Graphics2D g) {
    g.setColor(GRAY);
    g.fillRect(0, 80, 800, 520);

    for(int y = 0; y < 4; y++) {
      for(int x = 0; x < 4; x++) {
        if (tiles[y][x] < 15) {
          g.drawImage(Images.tiles[tiles[y][x]],
              140 + x * 130, 80 + y * 130, null);
        }
      }
    }

    if (movingSteps > 0) {
      g.drawImage(Images.tiles[movingTile], movingX, movingY, null);
    } else if (solvedCountdown > 0 && ((solvedCountdown >> 3) & 1) == 0) {
      g.setColor(Color.RED);
      g.fillRect(530, 470, 130, 130);
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {

    if (clickRegion.id == EXIT_REGION) {
      main.setMode(main.closetMode);
      addClickRegions();    
    } else {
      movingSteps = 5;
      targetX = holeX;
      targetY = holeY;
      switch(clickRegion.id) {
        case Images.UP:
          holeY--;
          movingVx = 0;
          movingVy = 26;
          break;
        case Images.DOWN:
          holeY++;
          movingVx = 0;
          movingVy = -26;
          break;
        case Images.LEFT:
          holeX--;
          movingVx = 26;
          movingVy = 0;
          break;
        case Images.RIGHT:
          holeX++;
          movingVx = -26;
          movingVy = 0;
          break;
      }
      movingTile = tiles[holeY][holeX];
      tiles[holeY][holeX] = 15;
      movingX = 140 + holeX * 130;
      movingY = 80 + holeY * 130;
    }
  }

  @Override
  public boolean isInventoryAccessible() {
    return solvedCountdown == 0;
  }
}
