package escape;

import java.awt.*;
import java.util.*;

public class VentMode extends GameMode {

  public int[][] vents = new int[3][4]; // 3 rooms, 4 vents each
  public int[] rightWays = new int[3]; // 3 correct choices

  public Region[] ventRegions = new Region[4];

  public boolean inTunnel;
  public int tunnelIndex;
  public int roomIndex;
  
  public VentMode() {
    randomizeVents();
    initializeVentRegions();
    reset();        
  }

  public void reset() {
    inTunnel = true;
    tunnelIndex = Images.RIGHT;
    roomIndex = 0;    
    addVentRegion();
  }
  
  public void initializeVentRegions() {
    for(int i = 0; i < 4; i++) {
      ventRegions[i] = new Region(110 + i * 150, 301, 130, 88, i,
          Images.arrowCursors[Images.UP]);
    }
  }

  public void addVentRegion() {
    clickRegions.add(new Region(300, 240, 200, 200, 0,
        Images.arrowCursors[tunnelIndex]));
    if (roomIndex == 3 && !GameState.acquiredKeycard) {
      clickRegions.add(new Region(344, 519, 137, 42, 10));
    }
  }

  public void addVentRegions() {
    for(int i = 0; i < 4; i++) {
      clickRegions.add(ventRegions[i]);
    }
  }

  public void randomizeVents() {
    ArrayList<Integer> list = new ArrayList<Integer>();
    ArrayList<Integer> list2 = new ArrayList<Integer>();
    for(int j = 0; j < 4; j++) {
      list2.add(j);
    }
    for(int i = 0; i < 3; i++) {
      for(int j = 0; j < 4; j++) {
        list.add(j);
      }
      for(int j = 0; j < 4; j++) {
        vents[i][j] = list.remove(random.nextInt(list.size()));
      }
      rightWays[i] = list2.remove(random.nextInt(list2.size()));
    }
  }

  @Override
  public void modify() {
  }

  @Override
  public void draw(Graphics2D g) {
    if (inTunnel) {
      g.drawImage(Images.largeVents[tunnelIndex], 0, 80, null);
      if (roomIndex == 3 && !GameState.acquiredKeycard) {
        g.drawImage(Images.keycards[1], 344, 519, null);
      }
    } else {
      g.drawImage(Images.ventRooms[roomIndex], 0, 80, null);
      for(int i = 0; i < 4; i++) {
        g.drawImage(Images.smallVents[vents[roomIndex][i]],
            100 + i * 150, 291, null);
      }
    }
  }

  @Override
  public void regionClicked(Region clickRegion) {

    if (clickRegion.id == 10) {
      GameState.acquiredKeycard = true;
      addToInventory(Images.keycards[0]);
      addVentRegion();
    } else if (roomIndex == 3) {
      Main.main.setMode(Main.main.trashCanMode);
    } else if (inTunnel) {
      inTunnel = false;      
      addVentRegions();
    } else {
      tunnelIndex = vents[roomIndex][clickRegion.id];
      if (rightWays[roomIndex] == clickRegion.id) {
        roomIndex++;
      } else {
        roomIndex = 0;
      }

      inTunnel = true;      
      addVentRegion();
    }
  }

}
